## Start by loading the dbatools module.
Import-Module dbatools;

## We can ask dbatools to give us a list of the commands available within the module.
Find-DbaCommand;
(Find-DbaCommand).Count;

## Let's connect to an instance and execute a query.
$ali3 = Connect-DbaInstance -SqlInstance ali3\sql2019 -SqlCredential sqlsaturday;
Invoke-DbaQuery -SqlInstance $ali3 -Query "select getdate() as CurrentTime;"

## List instances dbatools is presently connected to.
Get-DbaConnectedInstance;

## If we wanted to disconnected, we can do that.
# $ali3 | Disconnect-DbaInstance;

## A couple of commands just to show some information we can collect from the instance.
Get-DbaInstanceInstallDate $ali3;

Get-DbaInstanceProperty -SqlInstance $ali3 `
    -InstanceProperty SqlInstance, Edition, ProductLevel, VersionString, HostDistribution, OSVersion, PhysicalMemory, Processors |
    Out-GridView;

## List out the databases on the instance
Get-DbaDatabase -SqlInstance $ali3;

Get-DbaDatabase -SqlInstance $ali3 | Out-GridView;

## Many parameters can accept arrays. Let's connect to a second instance.
$ali2 = Connect-DbaInstance -SqlInstance ali2\sql2019 -SqlCredential sqlsaturday;

Get-DbaDatabase -SqlInstance $ali3, $ali2 | Out-GridView;

## Get a specific database
$db = Get-DbaDatabase -SqlInstance $ali2 -Database CorpDB;
$db.GetType().FullName;
$db;
$db | Select-Object *;

## Copy a database from one instance to another
$ali2 = Connect-DbaInstance -SqlInstance ali2\sql2019 -SqlCredential sqlsaturday;
$ali3 = Connect-DbaInstance -SqlInstance ali3\sql2019 -SqlCredential sqlsaturday;
Copy-DbaDatabase -Source $ali2 -Destination $ali3 -Database CorpDB -BackupRestore -SharedPath "\\ali\temp";

## Instance migration
Start-DbaMigration -Source $ali2 -Destination $ali3 -BackupRestore -SharedPath "\\ali\temp";

## Backup/restore
Backup-DbaDatabase -SqlInstance $ali3 -SqlCredential sqlsaturday -Database CorpDB -Path "c:\data\sql2019\backup";
Get-DbaLastBackup -SqlInstance $ali3 -SqlCredential sqlsaturday -Database CorpDB;
Remove-DbaBackup -BackupFileExtension "bak" -Path "\\ali3\c$\data\sql2019\backup" -RetentionPeriod "72h";

## Instance enumeration
Find-DbaInstance -ComputerName ali3 -ScanType Browser;

## Reading and writing from SQL Server
Invoke-DbaQuery -SqlInstance $ali3 -SqlCredential sqlsaturday -Database CorpDB `
    -Query "select top (100) * from CorpDB.dbo.Customer;";
Invoke-DbaQuery -SqlInstance $ali3 -SqlCredential sqlsaturday -Database CorpDB `
    -Query "select top (100) * from CorpDB.dbo.Customer;" | Out-GridView;
$customers = Invoke-DbaQuery -SqlInstance $ali3 -SqlCredential sqlsaturday -Database CorpDB `
    -Query "select top (100) * from CorpDB.dbo.Customer;";
Write-DbaDataTable -SqlInstance $ali3 -Database "tempdb" -InputObject $customers `
    -AutoCreateTable -Table "customer";

## Inventory databases
$databases = Get-DbaDatabase -SqlInstance $ali3;
Write-DbaDataTable -SqlInstance $ali3 -Database "tempdb" -InputObject $databases `
    -AutoCreateTable -Table "databases";

Get-DbaDatabase -SqlInstance $ali3 | Write-DbaDataTable -SqlInstance $ali3 -Database "tempdb" `
    -AutoCreateTable -Table "databases";
